#!/usr/bin/env python3
#-*- coding: utf-8 -*-
import os
from . import plentyofbugs as pob
from . import get_n_genomes as gng

#####  Pob tests
def test_get_best_mash_result():
    mash_results = [
        ['./genomes_of_niss/NC_007350.1.fasta', 'tmp/assembly/contigs.fasta', '0.3', '1', '0/1000'],
        ['./genomes_of_niss/NZ_CP014057.2.fasta', 'tmp/assembly/contigs.fasta', '0.6', '1', '0/1000'],
        ['./genomes_of_niss/NZ_CP014113.2.fasta', 'tmp/assembly/contigs.fasta', '.99', '1', '0/1000'],
        ['./genomes_of_niss/NZ_CP022056.1.fasta', 'tmp/assembly/contigs.fasta', '1', '1', '0/1000'],
        ['./genomes_of_niss/NZ_CP022093.1.fasta', 'tmp/assembly/contigs.fasta', '.15', '1', '0/1000'],
        ['./genomes_of_niss/NZ_LT963436.1.fasta', 'tmp/assembly/contigs.fasta', '.004', '1', '0/1000'],
        ['']
    ]
    test_result = pob.get_best_mash_result(mash_results)
    print(test_result)
    assert ["./genomes_of_niss/NZ_LT963436.1.fasta", .004] == test_result

###########################    get n genomes tests #####################
proks_file = "test_proks.txt"

def test_download_proks():
    if not os.path.exists(proks_file):
        gng.fetch_prokaryotes(dest=proks_file)
    assert os.path.exists(proks_file),  "error downloading prrokaryotes.txt"

def test_get_lines():
    org = "Escherichia coli"
    lines = gng.get_lines_of_interest_from_proks(path=proks_file,
                                                      org=org)
    print(lines)
    assert "PRJNA315511" in  [x[2] for x in lines],  "PRJNA315511 not found!!"

def test_get_names():
    org = "Escherichia coli"
    lines = gng.get_lines_of_interest_from_proks(path=proks_file,
                                                      org=org)
    names = []
    for line in lines:
        name = gng.parse_name_from_proks_line(line)
        print(name)
        names.append(name)

    assert "CP017061.1" in names,  "CP017061.1 not found!!"



"""
Setup for PlentyOfBugs
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
import re
from codecs import open
from os import path
import sys

# try: # for pip >= 10
#     from pip._internal.req import parse_requirements
# except ImportError: # for pip <= 9.0.3
#     from pip.req import parse_requirements

here = path.abspath(path.dirname(__file__))

VERSIONFILE = "pob/_version.py"
verstrline = open(VERSIONFILE, "rt").read()
VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
mo = re.search(VSRE, verstrline, re.M)
if mo:
    verstr = mo.group(1)
else:
    raise RuntimeError("Unable to find version string in %s." % (VERSIONFILE,))


if sys.version_info <= (3, 0):
    sys.stderr.write("ERROR: PlentyOfBugs requires Python 3.5 " +
                     "or above...exiting.\n")
    sys.exit(1)

## parse requirements file
# install_reqs = parse_requirements("requirements.txt",
#                                   session=False)
# requirements = [str(ir.req) for ir in install_reqs]

setup(
    name='PlentyofBugs',
    version=verstr,

    description='find your newly sequenced isolate the perfect companion reference genome',
    # long_description=long_description,
    long_description="""
    check out the GitHub
    repo for the real README.md file
    """,

    url='https://github.com/nickp60/plentyofbugs',

    # Author details
    author='Nick Waters',
    author_email='nickp60@gmail.com',
    license='MIT',
    # handle requirments
    install_requires=[],
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    keywords='bioinformatics evolution genomics development',
    packages=find_packages(),
    include_package_data=True,
    # package_data={
    #    '': [path.join(__name__, "tests", "refs/*")],
    # },
    entry_points={
       'console_scripts': [
           'plentyofbugs=pob.plentyofbugs:main',
           'get_n_genomes=pob.get_n_genomes:main',
       ],
    },
)



#!/usr/bin/env python3
#-*- coding: utf-8 -*-

"""
"""

import argparse
import sys
import time
import os
import traceback
import shutil
import subprocess

from . import get_n_genomes
from . import _version

# --------------------------- methods --------------------------- #


def get_args(test_args=None):  # pragma: no cover
    parser = argparse.ArgumentParser(
        description="Get the closest reference genome for an isolate ")
    parser.add_argument(
        "-f", "--readsF", action="store",
        help="forward reads", required=False)
    parser.add_argument(
        "-o", "--output", action="store",
        help="output directory", required=True)
    parser.add_argument(
        "-r", "--readsR", action="store",
        help="reverse reads", required=False)
    parser.add_argument(
        "-n", "--nstrains", action="store",
        type=int,
        help="number of strains", required=False)
    parser.add_argument(
        "--genus_species", dest="organism_name", action="store",
        help="'Genus species', in quotes", required=False)
    parser.add_argument(
        "--assembler", action="store",
        choices=["skesa", "spades"],
        default="skesa",
        help="assembler to use", required=False)
    parser.add_argument(
        "-m", "--method", action="store",
        choices=["pyani", "mash"],
        default="mash",
        dest="method",
        help="method to calculate genomic distances; use pyani " +
        "for accuracy, and mash for speed", required=False)
    parser.add_argument(
        "-p",
        "--prokaryotes",
        action="store",
        default="prokaryotes.txt",
        help="path_to_prokaryotes.txt",
        required=False)
    parser.add_argument(
        "-g",
        "--genomes_dir",
        help="path to genomes dir. Creates " +
        "it if it  doesnt exist",
        required=False)
    parser.add_argument(
        "-a",
        "--assembly",
        help="path to assembly",
        required=False)
    parser.add_argument(
        "-t",
        "--threads",
        default=4,
        type=int,
        help="number of threads to try to use",
        required=False)
    parser.add_argument(
        "--downsampling_ammount",
        help="downsampling ammount must be a float between 0.1 and .9999, or a number of reads",
        type=float,
        default=.9999,
        required=False)
    if test_args is None:
        args = parser.parse_args(sys.argv[1:])
    else:
        args = parser.parse_args(test_args)
    if args.genomes_dir is None:
        args.genomes_dir = "plentyofbugs_genomes"
    if not os.path.isdir(args.genomes_dir):
        os.makedirs(args.genomes_dir)
    if args.downsampling_ammount < .1:
        raise ValueError(args.downsampling_ammount, "invalid value")
    if args.downsampling_ammount > 1 and args.downsampling_ammount < 10:
        raise ValueError(args.downsampling_ammount, "invalid value")
    if args.method == "pyani":
        print("Error: please run PlentyOfBugs via a docker version < 0.88 to use pyani")
        sys.exit(1)
    if len(os.listdir(args.genomes_dir)) == 0 :
        if  args.organism_name is None:
            print("Error: if genomes_dir is empty, must provide --genus_species")
            sys.exit(1)
        if args.nstrains is None:
            print("Error: if genomes_dir is empty, must provide --nstrains")
            sys.exit(1)
    if args.readsF is None and args.assembly is None:
        print("Error: Must provide either --readsF or --assembly!")
        sys.exit(1)
    return args


def get_closest_ANI(path):
    matrix = []
    with open(sys.argv[1], 'r') as inf:
        for line in inf:
            thisline = line.strip().split("\t")
            matrix.append(thisline)
        # for i,x in enumerate(thisline):
        #     if x == "contigs" or x == "scaffolds":
                # print(i + 2) # unix indexes at 1, and line starts with a blak tab
                #                 sys.exit()

    # raise ValueError("contigs not found in header")
    # just start by getting the first forward comparison (ie, the vertical part of the matrix
    # we assume rthere is just one entry

    # lets transpose this
    tmatrix = list(map(list, zip(*matrix)))
    # print(tmatrix)

    # get the index of the row/column with id "contigs*
    contigs_idx = [i for i, x in enumerate(tmatrix) if x[0].startswith("contigs") or x[0].startswith("run")][0]
    # print(contigs_idx)

    # select our headers
    headers = matrix[0]
    # note here that we have to trim the first column from the row of interest, we have one less header column than the rest of the rows, cause the first one is empty
    line_of_interest = sorted(zip(headers, tmatrix[contigs_idx][1:]), key=lambda x: x[1], reverse=True)
    # print(line_of_interest)
    # bam
    # we need to do this stupid loop thing because if there is a score tie, the contigs
    # entry won't neccesdarily be first.  So, we go through the line, printing the first entry that doesnt start with "contigs" and isn't identical (probably an artifact)
    for pair in line_of_interest:
        # the new version appends _## to the names
        if not (pair[0].startswith("contigs") or pair[0].startswith("run")) and pair[1] != "1":
            closest_id = "_".join(pair[0].split("_")[0:-1])
            closest_id_perc = pair[1]
            return("{0}\t{1}".format(closest_id, closest_id_perc))


def get_best_mash_result(mash_results):
    # be aware there is an empty line at the end
    max_org = [[float(x[2]), x[0]] for x in mash_results[0:-1]]
    max_org.sort(reverse=False)
    return([max_org[0][1], max_org[0][0]])


def main():
    print("PlentyOfBugs version " + _version.__version__)
    # check for required programs
    args = get_args()
    args.output = os.path.join(args.output, "")
    for prog in [args.assembler, "mash", "wget"]:
        if shutil.which(prog) is None:
            print(prog + "not found in PATH! exiting...")
            sys.exit(1)
    if os.path.exists(args.output):
        print("Output directory already exists! exiting...")
        sys.exit(1)
    else:
        os.makedirs(args.output)
    #######################   Download close genomes  ###########################
    if len(os.listdir(args.genomes_dir))  == 0:
        get_n_genomes.main(args)
    #############################   Run Mini Assembly  #########################
    if args.assembly is None:
        # downsample
        cmds = []
        ds_dir = os.path.join(args.output,"downsampled_reads")
        ds_readsF = os.path.join(args.output,"downsampled_reads", "reads1.fq")
        os.makedirs(ds_dir)
        ds_cmd_1 = str(
            "seqtk sample -s100 {args.readsF}  {args.downsampling_ammount} " +
            "> {ds_readsF}").format(**locals())
        cmds.append(ds_cmd_1)
        if args.readsR is not None:
            ds_readsR = os.path.join(args.output,"downsampled_reads", "reads2.fq")
            ds_cmd_2 = str(
                "seqtk sample -s100 {args.readsR}  {args.downsampling_ammount} " +
                "> {ds_readsR}").format(**locals())
            cmds.append(ds_cmd_2)
            SPADES_STRING="-1 {ds_readsF} -2 {ds_readsR}".format(**locals())
            SKESA_STRING="{ds_readsF} {ds_readsR}".format(**locals())
        else:
            SPADES_STRING="-s {ds_readsF} ".format(**locals())
            SKESA_STRING="{ds_readsF}".format(**locals())
        assembly_dir = os.path.join(args.output, "assembly")
        args.assembly = os.path.join(assembly_dir, "contigs.fasta")
        if args.assembler == "skesa":
            os.makedirs(assembly_dir)
            assembly_cmd = str(
                "skesa --fastq {SKESA_STRING} --cores {args.threads} " +
                "--contigs_out {args.assembly} 2> " +
                "{args.output}/log.txt").format(**locals())
        else:
            assembly_cmd = str(
                "spades.py {SPADES_STRING} -o {assembly_dir} -t " +
                "{args.threads}").format(**locals())
        cmds.append(assembly_cmd)
        for cmd in cmds:
            print(cmd)
            subprocess.run(
                cmd,
                shell=sys.platform!="win32",
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                check=True)

    #######################   Find closest  ###########################

    if args.method == "pyani":
        pyani_cmds = []
        labels_txt = os.path.join(args.genomes_dir, "labels.txt")
        classes_txt = os.path.join(args.genomes_dir, "classes.txt")
        for path in ["labels.txt", "classes.txt"]:
            if os.path.exists(path):
                os.remove(os.path.join(args.genomes_dir, path))
        pyani_index = "pyani index {args.genomes_dir} -v"
        pyani_cmds.append(pyani_index)
        # then fix the columns; we dont care about the name
        fix_labels_cmds = [
            "paste <(cut -f 1,2 $GENOMESDIR/classes.txt) <(cut -f 2 $GENOMESDIR/classes.txt) > tmp_classes",
            "mv tmp_classes $GENOMESDIR/classes.txt"
            "paste <(cut -f 1,2 $GENOMESDIR/labels.txt) <(cut -f 2 $GENOMESDIR/labels.txt) > tmp_labels",
            "mv tmp_labels $GENOMESDIR/labels.txt"
        ]
        pyani_cmds.extend(fix_labels_cmds)
        pyanidb = os.path.join(args.genomes_dir, "pyani", "pyanidb")
        if not os.path.exists(pyanidb):
            pyani_cmds.append("pyani createdb --dbpath ${GENOMESDIR}/.pyani/pyanidb")
        pyani_cmds.append(str(
            "pyani anim {args.genomes_dir} {args.output}pyani --workers {args.threads}  --dbpath {pyanidb} -v --labels {labels_txt} --classes {classes_txt}").format(**locals())
                          )
        pyani_cmds.append("tail -n 1 $OUTDIR/pyani/runs.tab | cut -f 1")
        pyani_cmds.append(str(
            "pyani report -v --runs {args.output}pyani  --dbpath {pyanidb}").format(**locals()))

        for cmd in pyani_cmds:
            print(cmd)
            subprocess.run(cmd,
                shell=sys.platform!="win32",
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                check=True)
        ############# remove contigs from genomes dir if we plan on reusing   ########
        # get rid of the fasta and md5
        for path in glob.glob(os.path.join(args.genomes_dir, "contigs_")):
            os.remove(path)
        parse_closest_ANI(path)
    else:
        mash_cmds = []
        #
        mash_sketch = os.path.join(args.genomes_dir, "reference.msh")
        if not os.path.exists(mash_sketch):
            mash_cmds.append(
                "mash sketch -o {mash_sketch} {args.genomes_dir}*.f*a".format(
                    **locals()))
        mash_cmds.append("mash dist {mash_sketch} {args.assembly}".format(
            **locals()))
        cmds_results = mash_cmds
        for i, cmd in enumerate(mash_cmds):
            print(cmd)
            cmds_results[i] = subprocess.run(
                cmd,
                shell=sys.platform!="win32",
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                check=True)
        mash_results = [x.split("\t") for x in cmds_results[-1].stdout.decode().split("\n")]
        best_reference = get_best_mash_result(mash_results)
    with open(os.path.join(args.output, "best_reference"), "w") as outf:
        print("\t".join([str(x) for x in best_reference]))
        outf.write("\t".join([str(x) for x in best_reference]) + "\n")




if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: Nick Waters
"""

import os
import sys
import subprocess
import argparse
import shutil
from random import shuffle

def get_args():  # pragma nocover
    parser = argparse.ArgumentParser(
        description="Download a number of complete genomes for a given organism ",
        add_help=True)
    parser.add_argument(
        "-o",
        "--organism_name",
        help="organism_name",
        required=True)
    parser.add_argument(
        "-g",
        "--genomes_dir",
        help="path to output dir",
        required=True)
    parser.add_argument(
        "-n", "--nstrains",
        help="",
        type=int,
        required=True)

    parser.add_argument(
        "-p",
        "--prokaryotes",
        action="store",
        help="path_to_prokaryotes.txt",
        required=True)
    return(parser.parse_args())


def fetch_prokaryotes(dest):
    subprocess.run(
        "wget ftp://ftp.ncbi.nlm.nih.gov/genomes/GENOME_REPORTS/prokaryotes.txt -O " + dest,
        shell=sys.platform != "win32",
        check=True)


def parse_name_from_proks_line(line):
    seq_name = line[8].split(":")[1].split(";")[0].split("/")[0]
    return seq_name

def make_fetch_cmds(org_lines, nstrains, genomes_dir, SHUFFLE=True):
    if SHUFFLE:
        shuffle(org_lines)
    # # now we shuffle the file, get the top n lines, and  split apart the
    # # chromosome:NZ_CP013218.1/CP013218.1; plasmid paadD:NZ_CP014695.1/CP014695.1; plasmid plinE154:NZ_CP014694.1/CP014694.1
    # # to
    # # NZ_CP013218.1
    # # Note that we only get the first chromasome for a given entry. Sorry vibrioists
    #for line in org_lines:
    #    print(line[8].split(":")[1].split(";")[0].split("/")[0])
    cmds = []
    for line in org_lines[0:nstrains]:
        shortname = parse_name_from_proks_line(line)
        name = os.path.basename(line[20])
        full_path = os.path.join(
            line[20],
            name + "_genomic.fna.gz")
        cmd = "wget " + full_path + " -O " + os.path.join(
            genomes_dir, shortname + ".fna.gz")
        cmds.append(cmd)
    return cmds


def get_lines_of_interest_from_proks(path,  org):
    # column 9 has the nucc accession if it is a complete genome, or a "-" if empt
    # it starts with chromasome
    # here we select the lines for all the complete genomes,
    # find the lines matching out query
    # and save a list with the results
    org_lines = []
    with open(path, "r") as proks:
        for line in proks:
            splitline = line.strip().split("\t")
            if splitline[8].startswith("chrom"):
                if splitline[0].startswith(org):
                    org_lines.append(splitline)
    # for line in org_lines:
    #    print(line[20])
    # # if file is empty, raise an error
    if len(org_lines) == 0:
        print("no " + org +
              " matches in the prokaryotes.txt file")
    return org_lines

def main(args):
    if not os.path.exists(args.prokaryotes):
        fetch_prokaryotes(dest=args.prokaryotes)
    org_lines = get_lines_of_interest_from_proks(path=args.prokaryotes,
                                                 org=args.organism_name)

    cmds = make_fetch_cmds(
        org_lines,
        nstrains=args.nstrains,
        genomes_dir=args.genomes_dir,
        SHUFFLE=True)
    for cmd in cmds:
        print(cmd)
        subprocess.run(
            cmd,
            shell=sys.platform != "win32",
            check=True)

    unzip_cmd = "gunzip " + os.path.join(args.genomes_dir, "*")
    print(unzip_cmd)
    subprocess.run(
        unzip_cmd,
        shell=sys.platform != "win32",
    check=True)


if __name__ ==  "__main__":
    args = get_args()
    try:
        os.makedirs(args.genomes_dir)
    except:
        print("genomes output directory already  exists!")
        sys.exit(1)
    main(args)
